searchData={"items":[{"type":"task","title":"mix protox.generate","doc":"Generate Elixir code from `.proto` files.\n\nExample:\n`mix protox.generate --output-path=lib/message.ex --include-path=. message.proto`\n\nThe generated file will be usable in any project as long as protox is declared\nin the dependencies (the generated file still needs functions from the protox runtime).\n\nYou can use the `--namespace` option to prepend a namespace to all generated modules.\n\nIf you have large protobuf files, you can use the `--multiple-files` option to generate\none file per module.\n\nFinally, you can pass the option `--keep-unknown-fields=false` to remove support of\nunknown fields.","ref":"Mix.Tasks.Protox.Generate.html"},{"type":"module","title":"Protox","doc":"Use this module to generate the Elixir structs corresponding to a set of protobuf definitions\nand to encode/decode instances of these structures.","ref":"Protox.html"},{"type":"module","title":"Elixit structs generation examples - Protox","doc":"From a set of files:\n    defmodule Dummy do\n      use Protox,\n        files: [\n          \"./defs/foo.proto\",\n          \"./defs/bar.proto\",\n          \"./defs/baz/fiz.proto\",\n        ]\n    end\n\nFrom a string:\n    defmodule Dummy do\n      use Protox,\n        schema: \"\"\"\n        syntax = \"proto3\";\n        package fiz;\n\n        message Baz {\n        }\n\n        message Foo {\n          map  b = 2;\n        }\n        \"\"\"\n    end\n\nThe generated modules respect the package declaration. For instance, in the above example,\nboth the `Fiz.Baz` and `Fiz.Foo` modules will be generated.","ref":"Protox.html#module-elixit-structs-generation-examples"},{"type":"module","title":"Encoding/decoding - Protox","doc":"For the rest of this module documentation, we suppose the following protobuf messages are defined:\n    defmodule Dummy do\n      use Protox,\n        schema: \"\"\"\n          syntax = \"proto3\";\n          package fiz;\n\n          message Baz {\n          }\n\n          enum Enum {\n            FOO = 0;\n            BAR = 1;\n          }\n\n          message Foo {\n            Enum a = 1;\n            map  b = 2;\n          }\n        \"\"\",\n        namespace: Namespace\n\n      use Protox,\n        schema: \"\"\"\n        syntax = \"proto3\";\n\n        message Msg {\n          map  msg_k = 8;\n        }\n        \"\"\"\n\n      use Protox,\n        schema: \"\"\"\n        syntax = \"proto3\";\n\n        message Sub {\n          int32 a = 1;\n        }\n        \"\"\"\n    end\n\nSee each function documentation to see how they are used to encode and decode protobuf messages.","ref":"Protox.html#module-encoding-decoding"},{"type":"function","title":"Protox.decode/2","doc":"Decode a binary into a protobuf message.","ref":"Protox.html#decode/2"},{"type":"function","title":"Examples - Protox.decode/2","doc":"iex> binary = <<66, 7, 8, 1, 18, 3, 102, 111, 111, 66, 7, 8, 2, 18, 3, 98, 97, 114>>\n    iex> {:ok, msg} = Protox.decode(binary, Msg)\n    iex> msg\n    %Msg{msg_k: %{1 => \"foo\", 2 => \"bar\"}}\n\n    iex> binary = <<66, 7, 8, 1, 18, 3, 102, 111, 66, 7, 8, 2, 18, 3, 98, 97, 114>>\n    iex> {:error, reason} = Protox.decode(binary, Msg)\n    iex> reason\n    %Protox.IllegalTagError{message: \"Field with illegal tag 0\"}","ref":"Protox.html#decode/2-examples"},{"type":"function","title":"Protox.decode!/2","doc":"Throwing version of `decode/2`.","ref":"Protox.html#decode!/2"},{"type":"function","title":"Protox.encode/1","doc":"Encode a protobuf message into IO data.","ref":"Protox.html#encode/1"},{"type":"function","title":"Examples - Protox.encode/1","doc":"iex> msg = %Namespace.Fiz.Foo{a: 3, b: %{1 => %Namespace.Fiz.Baz{}}}\n    iex> {:ok, iodata} = Protox.encode(msg)\n    iex> :binary.list_to_bin(iodata)\n    <<8, 3, 18, 4, 8, 1, 18, 0>>\n\n    iex> msg = %Namespace.Fiz.Foo{a: \"should not be a string\"}\n    iex> {:error, reason} = Protox.encode(msg)\n    iex> reason\n    %Protox.EncodingError{field: :a, message: \"Could not encode field :a (invalid field value)\"}","ref":"Protox.html#encode/1-examples"},{"type":"function","title":"Protox.encode!/1","doc":"Throwing version of `encode/1`.","ref":"Protox.html#encode!/1"},{"type":"function","title":"Protox.json_decode/3","doc":"","ref":"Protox.html#json_decode/3"},{"type":"function","title":"Errors - Protox.json_decode/3","doc":"This function returns a tuple `{:error, reason}` if:\n- `input` could not be decoded to JSON; `reason` is a `Protox.JsonDecodingError` error","ref":"Protox.html#json_decode/3-errors"},{"type":"function","title":"JSON library configuration - Protox.json_decode/3","doc":"The default library to decode JSON is [`Jason`](https://github.com/michalmuskala/jason).\nHowever, you can chose to use [`Poison`](https://github.com/devinus/poison):\n    iex> Protox.json_decode(\"{\\\"a\\\":\\\"BAR\\\"}\", Namespace.Fiz.Foo, json_decoder: Poison)\n    {:ok, %Namespace.Fiz.Foo{__uf__: [], a: :BAR, b: %{}}}\n\nYou can also use another library as long as it exports an `decode!` function. You can easily\ncreate a module to wrap a library that would not have this interface (like [`jiffy`](https://github.com/davisp/jiffy)):\n    defmodule Jiffy do\n      def decode!(input) do\n        :jiffy.decode(input, [:return_maps, :use_nil])\n      end\n    end","ref":"Protox.html#json_decode/3-json-library-configuration"},{"type":"function","title":"Protox.json_decode!/3","doc":"Throwing version of `json_decode/2`.","ref":"Protox.html#json_decode!/3"},{"type":"function","title":"Protox.json_encode/2","doc":"Export a proto3 message to JSON as IO data.","ref":"Protox.html#json_encode/2"},{"type":"function","title":"Errors - Protox.json_encode/2","doc":"This function returns a tuple `{:error, reason}` if:\n- `msg` could not be encoded to JSON; `reason` is a `Protox.JsonEncodingError` error","ref":"Protox.html#json_encode/2-errors"},{"type":"function","title":"Examples - Protox.json_encode/2","doc":"iex> msg = %Namespace.Fiz.Foo{a: :BAR}\n    iex> {:ok, iodata} = Protox.json_encode(msg)\n    iex> iodata\n    [\"{\", [\"\\\"a\\\"\", \":\", \"\\\"BAR\\\"\"], \"}\"]\n\n    iex> msg = %Sub{a: 42}\n    iex> {:ok, iodata} = Protox.json_encode(msg)\n    iex> iodata\n    [\"{\", [\"\\\"a\\\"\", \":\", \"42\"], \"}\"]\n\n    iex> msg = %Msg{msg_k: %{1 => \"foo\", 2 => \"bar\"}}\n    iex> {:ok, iodata} = msg |> Protox.json_encode()\n    iex> :binary.list_to_bin(iodata)\n    \"{\\\"msgK\\\":{\\\"2\\\":\\\"bar\\\",\\\"1\\\":\\\"foo\\\"}}\"","ref":"Protox.html#json_encode/2-examples"},{"type":"function","title":"JSON library configuration - Protox.json_encode/2","doc":"The default library to encode values (i.e. mostly to escape strings) to JSON  is [`Jason`](https://github.com/michalmuskala/jason).\nHowever, you can chose to use [`Poison`](https://github.com/devinus/poison):\n    iex> msg = %Namespace.Fiz.Foo{a: :BAR}\n    iex> Protox.json_encode(msg, json_encoder: Poison)\n    {:ok, [\"{\", [\"\\\"a\\\"\", \":\", \"\\\"BAR\\\"\"], \"}\"]}\n\nYou can also use another library as long as it exports an `encode!` function, which is expected to return objects as maps and `nil`\nto represent `null`.\nYou can easily create a module to wrap a library that would not have this interface (like [`jiffy`](https://github.com/davisp/jiffy)):\n    defmodule Jiffy do\n      defdelegate encode!(msg), to: :jiffy, as: :encode\n    end","ref":"Protox.html#json_encode/2-json-library-configuration"},{"type":"function","title":"Encoding specifications - Protox.json_encode/2","doc":"See https://developers.google.com/protocol-buffers/docs/proto3#json for the specifications\nof the encoding.","ref":"Protox.html#json_encode/2-encoding-specifications"},{"type":"function","title":"Protox.json_encode!/2","doc":"Throwing version of `json_encode/1`.","ref":"Protox.html#json_encode!/2"},{"type":"module","title":"Protox.Default","doc":"Default values of Protocol Buffers types.\n\nNote that generated structs contain a default/1 function to return the default\nvalue of a field using its name.","ref":"Protox.Default.html"},{"type":"function","title":"Protox.Default.default/1","doc":"Returns the default value of a Protocol Buffer type specified with an atom.","ref":"Protox.Default.html#default/1"},{"type":"function","title":"Examples - Protox.Default.default/1","doc":"iex> Protox.Default.default(:bool)\n    false\n\n    iex> Protox.Default.default(:string)\n    \"\"","ref":"Protox.Default.html#default/1-examples"},{"type":"module","title":"Protox.Encode","doc":"This module contains the functions necessary to encode protobuf messages.","ref":"Protox.Encode.html"},{"type":"function","title":"Protox.Encode.encode/1","doc":"Encode a protobuf message into IO data.","ref":"Protox.Encode.html#encode/1"},{"type":"function","title":"Example - Protox.Encode.encode/1","doc":"msg = %Fiz.Foo{a: 3, b: %{1 => %Fiz.Baz{}}}\n    {:ok, iodata} = Protox.Encode.encode(msg)","ref":"Protox.Encode.html#encode/1-example"},{"type":"function","title":"Protox.Encode.encode!/1","doc":"Throwing version of `encode/1`.","ref":"Protox.Encode.html#encode!/1"},{"type":"module","title":"Protox.Field","doc":"The definition of a protobuf field (e.g. tag, type, etc.).","ref":"Protox.Field.html"},{"type":"type","title":"Protox.Field.t/0","doc":"","ref":"Protox.Field.html#t:t/0"},{"type":"behaviour","title":"Protox.JsonLibrary","doc":"The behaviour to implement when wrapping a JSON library.","ref":"Protox.JsonLibrary.html"},{"type":"callback","title":"Protox.JsonLibrary.decode!/2","doc":"Should wrap any exception of the underlying library in Protox.JsonDecodingError.","ref":"Protox.JsonLibrary.html#c:decode!/2"},{"type":"callback","title":"Protox.JsonLibrary.encode!/2","doc":"Should wrap any exception of the underlying library in Protox.JsonEncodingError.","ref":"Protox.JsonLibrary.html#c:encode!/2"},{"type":"callback","title":"Protox.JsonLibrary.load/0","doc":"","ref":"Protox.JsonLibrary.html#c:load/0"},{"type":"protocol","title":"Protox.JsonMessageDecoder","doc":"This protocol makes possible to override the JSON decoding of a specific message.","ref":"Protox.JsonMessageDecoder.html"},{"type":"function","title":"Protox.JsonMessageDecoder.decode_message/2","doc":"","ref":"Protox.JsonMessageDecoder.html#decode_message/2"},{"type":"type","title":"Protox.JsonMessageDecoder.t/0","doc":"All the types that implement this protocol.","ref":"Protox.JsonMessageDecoder.html#t:t/0"},{"type":"protocol","title":"Protox.JsonMessageEncoder","doc":"This protocol makes possible to override the JSON encoding of a specific message.\n\nFor instance, it's possible to encode\n[Google.Protobuf.Duration](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#google.protobuf.Duration)\nas a string rather than\nan object as required by the [JSON encoding specification](https://developers.google.com/protocol-buffers/docs/proto3#json).","ref":"Protox.JsonMessageEncoder.html"},{"type":"function","title":"Protox.JsonMessageEncoder.encode_message/2","doc":"The contract of a message encoder.\n\n- `msg` is the protobuf message to encode\n- `json_encode` is the function to use in the protocol implementation to encode values to JSON","ref":"Protox.JsonMessageEncoder.html#encode_message/2"},{"type":"type","title":"Protox.JsonMessageEncoder.t/0","doc":"All the types that implement this protocol.","ref":"Protox.JsonMessageEncoder.html#t:t/0"},{"type":"module","title":"Protox.MergeMessage","doc":"This module provides a helper function to merge messages.","ref":"Protox.MergeMessage.html"},{"type":"function","title":"Protox.MergeMessage.merge/2","doc":"Singular fields of `msg` will be overwritten, if specified in `from`, except for\nembedded messages which will be merged. Repeated fields will be concatenated.\n\nNote that \"specified\" has a different meaning in protobuf 2 and 3:\n- 2: if the singular field from `from` is nil, the value from `msg` is kept\n- 3: if the singular field from `from` is set to the default value, the value from `msg` is\nkept. This behaviour matches the C++ reference implementation behaviour.\n\n- `msg` and `from` must be of the same type; or\n- either `msg` or `from` is `nil`: the non-nil message is returned; or\n- both are `nil`: `nil` is returned\n\n# Example\n    iex> r1 = %Protobuf2{a: 0, s: :ONE}\n    iex> r2 = %Protobuf2{a: nil, s: :TWO}\n    iex> Protox.MergeMessage.merge(r1, r2)\n    %Protobuf2{a: 0, s: :TWO}\n    iex> Protox.MergeMessage.merge(r2, r1)\n    %Protobuf2{a: 0, s: :ONE}","ref":"Protox.MergeMessage.html#merge/2"},{"type":"module","title":"Protox.Types","doc":"This module describes types that define a protobuf message.\n\nSee https://developers.google.com/protocol-buffers/docs/encoding#structure.","ref":"Protox.Types.html"},{"type":"type","title":"Protox.Types.kind/0","doc":"This type give more details on how a field is encoded.","ref":"Protox.Types.html#t:kind/0"},{"type":"type","title":"Protox.Types.label/0","doc":"This type gives more information on the field presence (if applicable).","ref":"Protox.Types.html#t:label/0"},{"type":"type","title":"Protox.Types.map_key_type/0","doc":"All types that can be used as a key in map field.","ref":"Protox.Types.html#t:map_key_type/0"},{"type":"type","title":"Protox.Types.tag/0","doc":"The wire type of a field: it tells how a field is encoded (32 or 64 bits scalar, repeated or\nvariable-length integer).","ref":"Protox.Types.html#t:tag/0"},{"type":"type","title":"Protox.Types.type/0","doc":"All types that can be stored in a protobuf message.","ref":"Protox.Types.html#t:type/0"},{"type":"type","title":"Protox.Types.wire_32bits/0","doc":"32 bits scalar.","ref":"Protox.Types.html#t:wire_32bits/0"},{"type":"type","title":"Protox.Types.wire_64bits/0","doc":"64 bits scalar.","ref":"Protox.Types.html#t:wire_64bits/0"},{"type":"type","title":"Protox.Types.wire_delimited/0","doc":"Repeated field.","ref":"Protox.Types.html#t:wire_delimited/0"},{"type":"type","title":"Protox.Types.wire_varint/0","doc":"Variable length integer.","ref":"Protox.Types.html#t:wire_varint/0"},{"type":"exception","title":"Protox.DecodingError","doc":"This error is thrown when a data could not be decoded.","ref":"Protox.DecodingError.html"},{"type":"exception","title":"Protox.EncodingError","doc":"This error is thrown when a message could not be encoded.","ref":"Protox.EncodingError.html"},{"type":"exception","title":"Protox.IllegalTagError","doc":"This error is thrown when decoding data with a field which tag is 0.","ref":"Protox.IllegalTagError.html"},{"type":"exception","title":"Protox.InvalidFieldAttribute","doc":"This error is thrown when a field is constructed with an invalid atribute.","ref":"Protox.InvalidFieldAttribute.html"},{"type":"exception","title":"Protox.JsonDecodingError","doc":"This error is thrown when a JSON payload could not be decoded to a protobuf message.","ref":"Protox.JsonDecodingError.html"},{"type":"exception","title":"Protox.JsonEncodingError","doc":"This error is thrown when a protobuf message could not be encoded to JSON.","ref":"Protox.JsonEncodingError.html"},{"type":"exception","title":"Protox.RequiredFieldsError","doc":"This error is thrown when encoding or decoding a Protobuf 2 message\nwith unset required fields (that is, that have the value `nil`).","ref":"Protox.RequiredFieldsError.html"},{"type":"extras","title":"Protox","doc":"# Protox\n\n[![Elixir CI](https://github.com/ahamez/protox/actions/workflows/elixir.yml/badge.svg)](https://github.com/ahamez/protox/actions/workflows/elixir.yml) [![Coverage Status](https://coveralls.io/repos/github/ahamez/protox/badge.svg?branch=master)](https://coveralls.io/github/ahamez/protox?branch=master) [![Hex.pm Version](http://img.shields.io/hexpm/v/protox.svg)](https://hex.pm/packages/protox) [![Hex Docs](https://img.shields.io/badge/hex-docs-brightgreen.svg)](https://hexdocs.pm/protox/) [![License](https://img.shields.io/hexpm/l/protox.svg)](https://github.com/ahamez/protox/blob/master/LICENSE)\n\n`protox` is an Elixir library to work with [Google's Protocol Buffers](https://developers.google.com/protocol-buffers), versions 2 and 3. It supports both binary and JSON encoding and decoding ([well-known types](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf) included, except the Any type for the time being).\n\nThe primary objective of `protox` is **reliability**: it uses [property based testing](https://github.com/alfert/propcheck) and has a [near 100% code coverage](https://coveralls.io/github/ahamez/protox?branch=master). Also, using [mutation testing](https://en.wikipedia.org/wiki/Mutation_testing) with the invaluable help of [Muzak pro](https://devonestes.com/muzak), the quality of the `protox` test suite has been validated. Therefore, `protox` [passes all the tests](#conformance) of the conformance checker provided by Google.\n\nIt's also easy to use: just point to the `*.proto` files or give the schema to the `Protox` macro, no need to generate any file! However, should you need to generate files, a mix task is [available](#files-generation).\n\nGiven the following protobuf definition, `protox` will generate a `Msg` struct:\n```proto\nmessage Msg{\n  int32 a = 1;\n  map  b = 2;\n}\n```\n\nYou can then interact with `Msg` like any Elixir structure:\n\n```elixir\niex> msg = %Msg{a: 42, b: %{1 => \"a map entry\"}}\niex> {:ok, iodata} = Msg.encode(msg)\niex> {:ok, iodata} = Msg.json_encode(msg)\n\niex> binary = # read binary from a socket, a file, etc.\niex> {:ok, msg} = Msg.decode(binary)\niex> json = # read json from a socket, file, etc.\niex> {:ok, msg} = Msg.json_decode(json)\n```\n\nYou can find [here](https://github.com/ahamez/protox/blob/master/test/example_test.exs) a more involved example with most types.","ref":"readme.html"},{"type":"extras","title":"Table of contents - Protox","doc":"- [Prerequisites](#prerequisites)\n- [Installation](#installation)\n- [Usage with a textual description](#usage-with-a-textual-description)\n- [Usage with files](#usage-with-files)\n- [Protobuf binary format](#protobuf-binary-format)\n- [Protobuf JSON format](#protobuf-json-format)\n- [Packages and  namespaces](#packages-and--namespaces)\n- [Specify import path](#specify-import-path)\n- [Unknown fields](#unknown-fields)\n- [Unsupported features](#unsupported-features)\n- [Implementation choices](#implementation-choices)\n- [Generated code reference](#generated-code-reference)\n- [Files generation](#files-generation)\n- [Conformance](#conformance)\n- [Types mapping](#types-mapping)\n- [Benchmarks](#benchmarks)\n- [Development](#development)\n- [Credits](#credits)","ref":"readme.html#table-of-contents"},{"type":"extras","title":"Prerequisites - Protox","doc":"- Elixir >= 1.12\n- protoc >= 3.0 *This dependency is only required at compile-time*\n  `protox` uses Google's `protoc` (>= 3.0) to parse `.proto` files. It must be available in `$PATH`.\n\n  👉 You can download it [here](https://github.com/google/protobuf) or you can install it with your favorite package manager (`brew install protobuf`, `apt install protobuf-compiler`, etc.).\n\n  ℹ️ If you choose to generate files, `protoc` won't be needed to compile these files.","ref":"readme.html#prerequisites"},{"type":"extras","title":"Installation - Protox","doc":"Add `:protox` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:protox, \"~> 1.7\"}]\nend\n```\n\nIf you plan to use the JSON encoding, you'll need to add [`Jason`](https://github.com/michalmuskala/jason) to your dependencies:\n\n```elixir\ndef deps do\n  [\n    {:protox, \"~> 1.7\"},\n    {:jason, \"~> 1.2\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage with an inlined textual description - Protox","doc":"The following example generates two modules: `Baz` and `Foo` from a textual description:\n\n```elixir\ndefmodule MyModule do\n  use Protox, schema: \"\"\"\n  syntax = \"proto3\";\n\n  message Baz {\n  }\n\n  message Foo {\n    int32 a = 1;\n    map  b = 2;\n  }\n  \"\"\"\nend\n```\n\nℹ️ The module in which the `Protox` macro is called is completely ignored and therefore does not appear in the names of the generated modules.","ref":"readme.html#usage-with-an-inlined-textual-description"},{"type":"extras","title":"Usage with files - Protox","doc":"Here's how to generate the modules from a set of files:\n\n```elixir\ndefmodule MyModule do\n  use Protox, files: [\n    \"./defs/foo.proto\",\n    \"./defs/bar.proto\",\n    \"./defs/baz/fiz.proto\",\n  ]\nend\n```","ref":"readme.html#usage-with-files"},{"type":"extras","title":"Protobuf binary format - Protox","doc":"","ref":"readme.html#protobuf-binary-format"},{"type":"extras","title":"Encode - Protox","doc":"Here's how to create and encode a new message to binary protobuf:\n\n```elixir\niex> msg = %Fiz.Foo{a: 3, b: %{1 => %Fiz.Baz{}}}\niex> {:ok, iodata} = Protox.encode(msg)\n```\nOr, with throwing style:\n```elixir\niex> iodata = Protox.encode!(msg)\n```\n\nIt's also possible to call `encode/1` and `encode!/1` directly on the generated structures:\n\n```elixir\niex> {:ok, iodata} = Fiz.Foo.encode(msg)\niex> iodata = Fiz.Foo.encode!(msg)\n```\n\nℹ️ Note that `encode/1` returns an [IO data](https://hexdocs.pm/elixir/IO.html#module-use-cases-for-io-data) for efficiency reasons. Such  IO data can be used\ndirectly with files or sockets write operations:\n```elixir\niex> {:ok, iodata} = Protox.encode(%Fiz.Foo{a: 3, b: %{1 => %Fiz.Baz{}}})\n[[[], <<18>>, <<4>>, \"\\b\", <<1>>, <<18>>, <<0>>], \"\\b\", <<3>>]\niex> {:ok, file} = File.open(\"msg.bin\", [:write])\n{:ok, #PID<0.1023.0>}\niex> IO.binwrite(file, iodata)\n:ok\n```\n\n👉 You can use [`:binary.list_to_bin/1`](https://erlang.org/doc/man/binary.html#list_to_bin-1) or [`IO.iodata_to_binary`](https://hexdocs.pm/elixir/IO.html#iodata_to_binary/1) to get a binary:\n\n```elixir\niex> %Fiz.Foo{a: 3, b: %{1 => %Fiz.Baz{}}} |> Protox.encode!() |> :binary.list_to_bin()\n<<8, 3, 18, 4, 8, 1, 18, 0>>\n```","ref":"readme.html#encode"},{"type":"extras","title":"Decode - Protox","doc":"Here's how to decode a message from binary protobuf:\n\n```elixir\niex> {:ok, msg} = Protox.decode(<<8, 3, 18, 4, 8, 1, 18, 0>>, Fiz.Foo)\n```\nOr, with throwing style:\n```elixir\niex> msg = Protox.decode!(<<8, 3, 18, 4, 8, 1, 18, 0>>, Fiz.Foo)\n```\n\nIt's also possible to call `decode/1` and `decode!/1` directly on the generated structures:\n\n```elixir\niex> {:ok, msg} = Fiz.Foo.decode(<<8, 3, 18, 4, 8, 1, 18, 0>>)\niex> msg = Fiz.Foo.decode!(<<8, 3, 18, 4, 8, 1, 18, 0>>)\n```","ref":"readme.html#decode"},{"type":"extras","title":"Protobuf JSON format - Protox","doc":"`protox` implements the [Google's JSON specification](https://developers.google.com/protocol-buffers/docs/proto3#json).","ref":"readme.html#protobuf-json-format"},{"type":"extras","title":"Encode - Protox","doc":"Here's how to encode a message to JSON, exported as IO data:\n\n```elixir\niex> msg = %Fiz.Foo{a: 42}\niex> {:ok, iodata} = Protox.json_encode(msg)\n{:ok, [\"{\", [\"\\\"a\\\"\", \":\", \"42\"], \"}\"]}\n```\n\nOr, with throwing style:\n\n```elixir\niex> msg = %Fiz.Foo{a: 42}\niex> iodata = Protox.json_encode!(msg)\n[\"{\", [\"\\\"a\\\"\", \":\", \"42\"], \"}\"]\n```\n\nIt's also possible to call `json_encode` and `json_encode!` directly on the generated structures:\n\n```elixir\niex> {:ok, iodata} = Fiz.Foo.json_encode(msg)\niex> iodata = Fiz.Foo.json_encode!(msg)\n```","ref":"readme.html#encode"},{"type":"extras","title":"Decode - Protox","doc":"Here's how to decode JSON to a message:\n\n```elixir\niex> Protox.json_decode(\"{\\\"a\\\":42}\", Fiz.Foo)\n{:ok, %Fiz.Foo{__uf__: [], a: 42, b: %{}}}\n```\n\nOr, with throwing style:\n\n```elixir\niex> Protox.json_decode!(\"{\\\"a\\\":42}\", Fiz.Foo)\n%Fiz.Foo{__uf__: [], a: 42, b: %{}}\n```\n\nIt's also possible to call `json_decode` and `json_decode!` directly on the generated structures:\n\n```elixir\niex> Fiz.Foo.json_decode(\"{\\\"a\\\":42}\")\niex> Fiz.Foo.json_decode!(\"{\\\"a\\\":42}\")\n```","ref":"readme.html#decode"},{"type":"extras","title":"JSON library configuration - Protox","doc":"By default, `protox` uses [`Jason`](https://github.com/michalmuskala/jason) to encode values to JSON (mostly to escape strings). You can also use [`Poison`](https://github.com/devinus/poison):\n\n```elixir\niex> Protox.json_decode!(iodata, Fiz.Foo, json_library: Protox.Poison)\niex> Protox.json_encode!(msg, json_library: Protox.Poison)\n```\n\nℹ️ You can use any other library by implementing the `Protox.JsonLibrary` behaviour.\n\n👉 Don't forget to add the chosen library to the list of dependencies in `mix.exs`.","ref":"readme.html#json-library-configuration"},{"type":"extras","title":"Well-known types - Protox","doc":"Note that `protox` does not completely support the [Any](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#any) well-know type: it will be encoded and decoded like a regular message, rather than with the custom encoding specified in the [JSON specification](https://developers.google.com/protocol-buffers/docs/proto3#json).","ref":"readme.html#well-known-types"},{"type":"extras","title":"Packages and  namespaces - Protox","doc":"","ref":"readme.html#packages-and-namespaces"},{"type":"extras","title":"Packages - Protox","doc":"Protobuf provides a `package` [directive](https://developers.google.com/protocol-buffers/docs/proto#packages):\n\n```proto\npackage abc.def;\nmessage Baz {}\n```\n\nModules generated by protox will include this package declaration. Thus, the example above will be translated to `Abc.Def.Baz` (note the [camelization](#implementation-choices) of package `abc.def` to `Abc.Def`).","ref":"readme.html#packages"},{"type":"extras","title":"Prepend namespaces - Protox","doc":"In addition, protox provides the possibility to prepend a namespace with the `namespace` option to all generated modules:\n\n```elixir\ndefmodule Bar do\n  use Protox, schema: \"\"\"\n    syntax = \"proto3\";\n\n    package abc;\n\n    message Msg {\n        int32 a = 1;\n      }\n    \"\"\",\n    namespace: MyApp\nend\n```\n\nIn this example, the module `MyApp.Abc.Msg` is generated:\n\n```elixir\niex> msg = %MyApp.Abc.Msg{a: 42}\n```","ref":"readme.html#prepend-namespaces"},{"type":"extras","title":"Specify import path - Protox","doc":"An import path can be specified using the `path:` or `paths:` options that respectively specify the directory or directories in which to search for imports:\n\n```elixir\ndefmodule Baz do\n  use Protox,\n    files: [\n      \"./defs/prefix/foo.proto\",\n      \"./defs/prefix/bar/bar.proto\",\n    ],\n    path: \"./defs\"\nend\n```\n\nIf multiple search paths are needed:\n\n```elixir\ndefmodule Baz do\n  use Protox,\n    files: [\n      \"./defs1/prefix/foo.proto\",\n      \"./defs1/prefix/bar.proto\",\n      \"./defs2/prefix/baz/baz.proto\"\n    ],\n    paths: [\n      \"./defs1\",\n      \"./defs2\"\n    ]\nend\n```\n\nIt corresponds to the `-I` option of `protoc`.","ref":"readme.html#specify-import-path"},{"type":"extras","title":"Unknown fields - Protox","doc":"[Unknown fields](https://developers.google.com/protocol-buffers/docs/proto3#unknowns) are fields that are present on the wire but which do not correspond to an entry in the protobuf definition. Typically, it occurs when the sender has a newer version of the protobuf definition. It enables backwards compatibility as the receiver with an old version of the protobuf definition will still be able to decode old fields.\n\nWhen unknown fields are encountered at decoding time, they are kept in the decoded message. It's possible to access them with the  `unknown_fields/1` function defined with the message.\n\n```elixir\niex> msg = Msg.decode!(<<8, 42, 42, 4, 121, 97, 121, 101, 136, 241, 4, 83>>)\n%Msg{a: 42, b: \"\", z: -42, __uf__: [{5, 2, <<121, 97, 121, 101>>}]}\n\niex> Msg.unknown_fields(msg)\n[{5, 2, <<121, 97, 121, 101>>}]\n```\n\nYou must always use `unknown_fields/1` as the name of the field (e.g. `__uf__` in the above example) is generated at compile-time to avoid collision with the actual fields of the Protobuf message. This function returns a list of tuples `{tag, wire_type, bytes}`. For more information, please see [protobuf encoding guide](https://developers.google.com/protocol-buffers/docs/encoding).\n\nWhen you encode a message that contains unknown fields, they will be reencoded in the serialized output.","ref":"readme.html#unknown-fields"},{"type":"extras","title":"Disable support of unknown fields - Protox","doc":"You can deactivate the support of unknown fields by setting the `:keep_unknown_fields` option to `false`:\n```elixir\ndefmodule Baz do\n  use Protox,\n    schema: \"\"\"\n    syntax = \"proto3\";\n\n    message Sub {\n      int32 a = 1;\n      string b = 2;\n    }\n    \"\"\",\n    keep_unknown_fields: false\nend\n```\nℹ️ `protox` will still correctly parse unknown fields, they just won't be added to the structure and you won't be able to access them. This also means that unkown fields won't be serialized back.","ref":"readme.html#disable-support-of-unknown-fields"},{"type":"extras","title":"Unsupported features - Protox","doc":"* The [Any](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#any) well-known type is partially supported: you can manually unpack the embedded message right after decoding and conversely pack it right before encoding;\n* Groups ([deprecated in protobuf](https://developers.google.com/protocol-buffers/docs/proto#groups));\n* All [options](https://developers.google.com/protocol-buffers/docs/proto3#options) other than `packed` and `default` are ignored as they concern other languages implementation details.","ref":"readme.html#unsupported-features"},{"type":"extras","title":"Implementation choices - Protox","doc":"* This library enforces the presence of required fields (Protobuf 2). Therefore an error is raised when encoding or decoding a message with a missing required field:\n    ```elixir\n    defmodule Bar do\n      use Protox, schema: \"\"\"\n        syntax = \"proto2\";\n\n        message Required {\n          required int32 a = 1;\n        }\n      \"\"\"\n    end\n\n    iex> Protox.encode!(%Required{})\n    ** (Protox.RequiredFieldsError) Some required fields are not set: [:a]\n\n    iex> Required.decode!(<<>>)\n    ** (Protox.RequiredFieldsError) Some required fields are not set: [:a]\n    ```\n\n* When decoding enum aliases, the last encountered constant is used. For instance, in the following example, `:BAR` is always used if the value `1` is read on the wire:\n    ```protobuf\n    enum E {\n      option allow_alias = true;\n      FOO = 0;\n      BAZ = 1;\n      BAR = 1;\n    }\n    ```\n\n* Unset optionals\n    * For Protobuf 2, unset optional fields are mapped to `nil`. You can use the generated `default/1` function to get the default value of a field:\n        ```elixir\n        defmodule Bar do\n          use Protox,\n          schema: \"\"\"\n            syntax = \"proto2\";\n\n            message Foo {\n              optional int32 a = 1 [default = 42];\n            }\n          \"\"\"\n        end\n\n        iex> Foo.default(:a)\n        {:ok, 42}\n\n        iex> %Foo{}.a\n        nil\n\n        ```\n        It means that if you need to know if a field has been set by the sender, you just have to test if its value is `nil` or not.\n\n    * For Protobuf 3, unset fields are mapped to their [default values](https://developers.google.com/protocol-buffers/docs/proto3#default). However, if you use the `optional` keyword (available in protoc version 3.15 and higher), then unset fields will be mapped to `nil`:\n        ```elixir\n        defmodule Bar do\n          use Protox,\n          schema: \"\"\"\n            syntax = \"proto3\";\n\n            message Foo {\n              int32 a = 1;\n              optional int32 b = 2;\n            }\n          \"\"\"\n        end\n\n        iex> Foo.default(:a)\n        {:ok, 0}\n\n        iex> %Foo{}.a\n        0\n\n        iex> Foo.default(:b)\n        {:error, :no_default_value}\n\n        iex> %Foo{}.b\n        nil\n        ```\n\n* Messages and enums names: they are converted using the [`Macro.camelize/1`](https://hexdocs.pm/elixir/Macro.html#camelize/1) function.\n  Thus, in the following example, `non_camel_message` becomes `NonCamelMessage`, but the field `non_camel_field` is left unchanged:\n    ```elixir\n    defmodule Bar do\n      use Protox,\n      schema: \"\"\"\n        syntax = \"proto3\";\n\n        message non_camel_message {\n        }\n\n        message CamelMessage {\n          int32 non_camel_field = 1;\n        }\n      \"\"\"\n    end\n\n\n    iex> msg = %NonCamelMessage{}\n    %NonCamelMessage{__uf__: []}\n\n    iex> msg = %CamelMessage{}\n    %CamelMessage{__uf__: [], non_camel_field: 0}\n    ```","ref":"readme.html#implementation-choices"},{"type":"extras","title":"Generated code reference - Protox","doc":"The detailed reference of the generated code is available [here](documentation/reference.md).","ref":"readme.html#generated-code-reference"},{"type":"extras","title":"Files generation - Protox","doc":"It's possible to generate a file that will contain all code corresponding to the protobuf messages:\n\n```shell\nMIX_ENV=prod mix protox.generate --output-path=/path/to/message.ex --include-path=./test/samples test/samples/messages.proto test/samples/proto2.proto\n```\n\nThe generated file will be usable in any project as long as `protox` is declared in the dependencies as it needs functions from the protox runtime.","ref":"readme.html#files-generation"},{"type":"extras","title":"Options - Protox","doc":"* `--output-path`\n  The path to the file to be generated or to the destination folder when generating multiple files.\n\n* `--include-path`\n  Specifies the [import path](#specify-import-path). If multiple include paths are needed, add more `--include-path` options.\n\n\n* `--multiple-files`\n  Generates one file per module. In this case, `--output-path` must point to a directory. It's useful for definitions with a lot of messages as Elixir will be able to parallelize the compilation of the generated modules.\n\n* `--namespace`\n  Prepends a [namespace](#prepend-namespaces) to all generated modules.\n\n* `--keep-unknown-fields=[true|false]`\n  Toggles support of [unknown fields](#unknown-fields). Default to `true`.\n\n* `--generate-defs-funs=[true|false]`\n  Generates deprecated functions `defs/0` and `defs_by_name/0`. Default to `true` for backward compatibility, but it's safe to set it to `false` if you don't use these functions (and it reduces the size of the generated code).","ref":"readme.html#options"},{"type":"extras","title":"Conformance - Protox","doc":"The protox library has been thoroughly tested using the conformance checker [provided by Google](https://github.com/protocolbuffers/protobuf/tree/master/conformance).\n\nHere's how to launch the conformance tests:\n\n* Get conformance-test-runner [sources](https://github.com/protocolbuffers/protobuf/archive/refs/tags/v3.18.0.tar.gz).\n* Compile conformance-test-runner ([macOS and Linux only](https://github.com/protocolbuffers/protobuf/tree/master/conformance#portability)):\n    ```\n    tar xf protobuf-3.18.0.tar.gz && cd protobuf-3.18.0 && ./autogen.sh && ./configure && make -j && cd conformance && make -j\n    ```\n\n* Launch the conformance tests:\n    ```\n    mix protox.conformance --runner=/path/to/protobuf-3.18.0/conformance/conformance-test-runner\n    ```\n* A report will be generated in the directory `conformance_report` and the following text should be displayed:\n\n    ```\n    CONFORMANCE TEST BEGIN ====================================\n\n    CONFORMANCE SUITE PASSED: 1996 successes, 0 skipped, 21 expected failures, 0 unexpected failures.\n\n\n    CONFORMANCE TEST BEGIN ====================================\n\n    CONFORMANCE SUITE PASSED: 0 successes, 120 skipped, 0 expected failures, 0 unexpected failures.\n    ```\n\n* You can alternatively launch these conformance tests with `mix test` by setting the `PROTOBUF_CONFORMANCE_RUNNER` environment variable and including the `conformance` tag:\n     ```\n     PROTOBUF_CONFORMANCE_RUNNER=/path/to/conformance-test-runner MIX_ENV=test mix test --include conformance\n     ```","ref":"readme.html#conformance"},{"type":"extras","title":"Skipped conformance tests - Protox","doc":"You may have noticed that there are `21 expected failures`. Indeed, we removed on purpose some conformance tests that `protox` can't currently pass. Here are the reasons why:\n\n- [Any](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#any) is not yet supported by `protox`;\n- We could not find the specification for the protobuf2 case of field name extensions when decoding from JSON.\n\nThe exact list of skipped tests is [here](https://github.com/ahamez/protox/blob/master/conformance/failure_list.txt).","ref":"readme.html#skipped-conformance-tests"},{"type":"extras","title":"Types mapping - Protox","doc":"The following table shows how Protobuf types are mapped to Elixir's ones.\n\nProtobuf   | Elixir\n---------- | -------------\n`int32`    | `integer()`\n`int64`    | `integer()`\n`uint32`   | `integer()`\n`uint64`   | `integer()`\n`sint32`   | `integer()`\n`sint64`   | `integer()`\n`fixed32`  | `integer()`\n`fixed64`  | `integer()`\n`sfixed32` | `integer()`\n`sfixed64` | `integer()`\n`float`    | `float() \\| :infinity \\| :'-infinity' \\| :nan`\n`double`   | `float() \\| :infinity \\| :'-infinity' \\| :nan`\n`bool`     | `boolean()`\n`string`   | `String.t()`\n`bytes`    | `binary()`\n`repeated` | `list(value_type)` where `value_type` is the type of the repeated field\n`map`      | `map()`\n`oneof`    | `{atom(), value_type}` where `atom()` is the type of the set field and where `value_type` is the type of the set field\n`enum`     | `atom() \\| integer()`\n`message`  | `struct()`","ref":"readme.html#types-mapping"},{"type":"extras","title":"Benchmarks - Protox","doc":"You can launch benchmarks to see how `protox` perform:\n```\nmix run ./benchmarks/generate_payloads.exs # first time only, generates random payloads\nmix run ./benchmarks/run.exs --lib=./benchmarks/protox.exs\nmix run ./benchmarks/load.exs\n```","ref":"readme.html#benchmarks"},{"type":"extras","title":"Development - Protox","doc":"`protox` uses [`pre-commit`](https://pre-commit.com/#install) to launch git hooks. Thus, it's strongly recommended to install it, and then to install hooks as follows:\n```sh\npre-commit install && pre-commit install -t pre-push\n```","ref":"readme.html#development"},{"type":"extras","title":"Credits - Protox","doc":"Both [gpb](https://github.com/tomas-abrahamsson/gpb) and [exprotobuf](https://github.com/bitwalker/exprotobuf) were very useful in understanding how to implement Protocol Buffers.","ref":"readme.html#credits"},{"type":"extras","title":"Generated code reference","doc":"# Generated code reference\n\nThis documentation lists the functions generated with each Elixir structure associated to a protobuf [message](documentation/reference.md#messages) or [enum](documentation/reference.md#enums).","ref":"reference.html"},{"type":"extras","title":"Messages - Generated code reference","doc":"","ref":"reference.html#messages"},{"type":"extras","title":"Encode - Generated code reference","doc":"```elixir\n@spec encode(struct) :: {:ok, iodata} | {:error, any}\nencode(msg)\n```\n\nEncode `msg` into an `iodata` suitable for files or sockets.\nReturns `{:ok, iodata}` when the encoding was successful and `{:error, description}` in case of an encoding error.\n\n\n```elixir\n@spec encode!(struct) :: iodata | no_return\nencode!(msg)\n```\n\nThrowing version of `encode/1`.","ref":"reference.html#encode"},{"type":"extras","title":"Decode - Generated code reference","doc":"```elixir\n@spec decode(binary) :: {:ok, struct} | {:error, any}\ndecode(data)\n```\n\nDecode binary `data` into an structure with the type of the module on which this function is called.\nReturns `{:ok, msg}` when the decoding was successful and `{:error, description}` in case of an decoding error.\n\n\n```elixir\n@spec decode!(binary) :: struct | no_return\ndecode!(data)\n```\n\nThrowing version of `decode/1`.","ref":"reference.html#decode"},{"type":"extras","title":"Syntax - Generated code reference","doc":"```elixir\n@spec syntax() :: atom\nsyntax()\n```\nGet the syntax of the protobuf message: `:proto2` or `:proto3`.","ref":"reference.html#syntax"},{"type":"extras","title":"Default values - Generated code reference","doc":"```elixir\n@spec default(atom) :: {:ok, boolean | integer | String.t() | binary | float} | {:error, atom}\ndefault(field_name)\n```\n\nGet the default value of a message field. Note that for Protobuf 3, the default value is mandated by [the Google reference documentation](https://developers.google.com/protocol-buffers/docs/proto3#default).","ref":"reference.html#default-values"},{"type":"extras","title":"Unknown fields - Generated code reference","doc":"_Note_: these functions are not available when the option `:keep_unknown_fields` is set to `false`.\n\n```elixir\n@spec clear_unknown_fields(struct) :: struct\nclear_unknown_fields(msg)\n```\nReturns a copy of `msg` with all unknown fields removed.\n\n```elixir\n@spec unknown_fields(struct) :: [{non_neg_integer, Protox.Types.tag(), binary}]\nunknown_fields(msg)\n```\nGet the unknown fields that may have been encountered when decoding data.\nSee [Types](documentation/reference.md#types) section to get a description of `Protox.Types.tag`.\n\n```elixir\n@spec unknown_fields_name() :: atom\nunknown_fields_name(msg)\n```\nGet the name of the field that stores unknown fields.","ref":"reference.html#unknown-fields"},{"type":"extras","title":"Required fields - Generated code reference","doc":"```elixir\n@spec required_fields() :: [atom]\nrequired_fields()\n```\nGet the list of required fields.","ref":"reference.html#required-fields"},{"type":"extras","title":"Metadata - Generated code reference","doc":"```elixir\n@spec file_options() :: struct() | nil\nfile_options()\n```\nGet the options set in `*.proto` files (see [the Google reference documentation](https://developers.google.com/protocol-buffers/docs/proto#options)). Returns a [`FileOption`](https://github.com/protocolbuffers/protobuf/blob/cac9765af0ace57ce00b6ea07b8829339a622b1d/src/google/protobuf/descriptor.proto#L346) structure. All messages of a same file share the same options.","ref":"reference.html#metadata"},{"type":"extras","title":"Enums - Generated code reference","doc":"```elixir\n@spec default() :: atom\ndefault()\n```\nGet the default value of an enum.\n\n```elixir\n@spec encode(atom) :: integer | atom\nencode(enum_entry)\n```\nGet the integer value of an enum entry. If `enum_entry` does not exist in the enum, it is returned as is.\n\n\n```elixir\n@spec decode(integer) :: atom | integer\ndecode(value)\n```\nGet the enum entry of an integer value. If `value` does not correpond to any entry in the enum, it is returned as is.\n\n```elixir\n@spec constants() :: [{integer, atom}]\nconstants()\n```\nGet the list of all the constants of the enum that correponds to the module on which this function has been called.","ref":"reference.html#enums"},{"type":"extras","title":"Types - Generated code reference","doc":"Types `Protox.Types.tag`, `Protox.Types.kind` and `Protox.Types.type` are defined as follows\n(see [here](https://developers.google.com/protocol-buffers/docs/encoding#structure) for more details):\n\n```elixir\n@type wire_varint :: 0\n@type wire_64bits :: 1\n@type wire_delimited :: 2\n@type wire_32bits :: 5\n\n@type tag :: wire_varint | wire_64bits | wire_delimited | wire_32bits\n@type kind :: {:scalar, any} | :packed | :unpacked | :map | {:oneof, atom}\n@type map_key_type ::\n        :int32\n        | :int64\n        | :uint32\n        | :uint64\n        | :sint32\n        | :sint64\n        | :fixed32\n        | :fixed64\n        | :sfixed32\n        | :sfixed64\n        | :bool\n        | :string\n@type type ::\n        :fixed32\n        | :sfixed32\n        | :float\n        | :fixed64\n        | :sfixed64\n        | :double\n        | :int32\n        | :uint32\n        | :sint32\n        | :int64\n        | :uint64\n        | :sint64\n        | :bool\n        | :string\n        | :bytes\n        | {:enum, atom}\n        | {:message, atom}\n        | {map_key_type, type}\n\n```","ref":"reference.html#types"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}